#pragma once
#include <ftlSharePtr.h>

//#define TRANSLATE_MAX_TRY_COUNT		3
//#define TRANSLATE_RETRY_TIME_SPAN	3

enum TransferError
{
	eSuccess = 0,
	eUserCancel,
	eNetwork,			
	eServer,
	eNoEnoughDisk,		
	eUnknown,
};


//dont't change the order
enum TransferStatus
{
	sInvalid = -1,

	sUploadWaiting,
	sUploading,
	sUploadSuccess,
	sUploadError,

	sRecvWaiting,
	sReceiving,
	sRecvSuccess,
	sRecvError,
	sUploadCancle,
	sRecvCancle,

    //sSendFileNotificationSuccess,
    //sSendFileNotificationFail,
};

//DWORD(GetLastError) -> TransferError;

class ITranslateItemCallBack;

class TranslateFileInfo : public CObject
{
public:
	DECLARE_SERIAL(TranslateFileInfo)
public:
	TranslateFileInfo();
	TranslateFileInfo(AttachFileInfo& fileInfo, BOOL bUpload);
	TranslateFileInfo(AttachFileInfo& fileInfo, const std::vector<wstring>& vecReceiverIds, BOOL bUpload);
	~TranslateFileInfo();
	void Clear();
	
	void ResetProgressInfo();
	void SetTranslateCallback(ITranslateItemCallBack* pCallBack);

	BOOL IsUploadJob() const { return m_bUpload; }
	TransferError  GetResult() const { return m_transferResult; }
	void SetResultByLastError(DWORD dwLastError);

	TransferStatus GetStatus() const { return m_Status; }
	void SetStatus(TransferStatus status);
	
	INT	GetProgress();
	void SetProgress(LONG nCurPos, LONG nTotalSize, BOOL bCalcSize);

	void Serialize( CArchive& archive );
	
public: //private
	AttachFileInfo				m_fileInfo;
	std::vector<wstring>		m_vecReceiverIds;
	BOOL						m_bUpload;
	INT							m_nJobIndex;
	UINT						m_TidRequestFileTranslate;
    UINT		                m_nFileSeq;
	CTime						m_tmTime;
	CTime						m_tmSendNotify;
	CString						m_strSenderID;
	CTime						m_tmFailTime;	//will try after 3 second
	//INT							m_nTryCount;	
	//Recv File
	int							m_nOsType;
	CString						m_strName;
	CString						m_strNickName;

	TransferStatus				m_Status;
	LONG						m_nCurPos;		//0 is init, 0~m_nTotalSize
	LONG						m_nTotalSize;	//0 is init

	INT							m_nLastProgress;
	DWORD						m_nLastTickCout;
	int							m_nSqliteSeq;
private:

	void InPut(CArchive &archive);
	void OutPut(CArchive &archive);

	TransferError				m_transferResult;
	
	ITranslateItemCallBack*		m_pCallBack;
	static TransferError ConvertTranslateError(DWORD dwLastError);
};
typedef CFSharePtr<TranslateFileInfo>	TranslateFileInfoPtr;

class ITranslateItemCallBack
{
public:
	virtual void OnTransferStart(TranslateFileInfo& fileInfo) = 0;
	virtual void OnTransferProgress(TranslateFileInfo& fileInfo, INT nProgress, BOOL bCalcSize) = 0;
	virtual void OnTransferError(TranslateFileInfo& fileInfo, TransferError err) = 0;
	virtual void OnTransferEnd(TranslateFileInfo& fileInfo, TransferStatus status) = 0;
};

